Module analysis

Module analysis 

E-graph analysis struct that helps “type check” rewrite rules.

Structs

LutAnalysis
The analysis struct allows for discovering when signals are equivalent to constants or leaf inputs. Additonally, the struct assists in folding constant inputs to smaller LUTs.
LutAnalysisData
An e-class is typically a boolean signal. However, we store constants and input aliases for folding. A lut::LutLang::Program should never really be rewritten, so storing programs allow us to quickly check if a class is a program and extract the program.