pub enum LutLang {
}Expand description
Definitions of e-node types. Programs are the only node type that is not a net/signal.
Variants
Const(bool)
Program(u64)
DC
Var(Symbol)
Nor([Id; 2])
Mux([Id; 3])
And([Id; 2])
Xor([Id; 2])
Not([Id; 1])
Lut(Box<[Id]>)
Bus(Box<[Id]>)
Reg([Id; 4])
Arg([Id; 1])
Cycle([Id; 1])
Implementations
Sourceimpl LutLang
impl LutLang
Sourcepub const MAX_LUT_SIZE: usize = 6
pub const MAX_LUT_SIZE: usize = 6
Maximum size allowed for a LUT.
This cannot be made larger due to us using u64 to represent LUTs. FPGAs generally do not
support greater than 6-LUTs so this limit should hopefully be sufficient forever.
Sourcepub fn verify_rec(&self, expr: &RecExpr<Self>) -> Result<(), String>
pub fn verify_rec(&self, expr: &RecExpr<Self>) -> Result<(), String>
Recursively verify the grammar of a LutLang expression expr rooted at self
Sourcepub fn get_program(&self, expr: &RecExpr<Self>) -> Result<u64, String>
pub fn get_program(&self, expr: &RecExpr<Self>) -> Result<u64, String>
Extract the program from a LutLang::Lut contained in expression expr
Sourcepub fn get_program_in_egraph(
&self,
egraph: &EGraph<LutLang, LutAnalysis>,
) -> Result<u64, String>
pub fn get_program_in_egraph( &self, egraph: &EGraph<LutLang, LutAnalysis>, ) -> Result<u64, String>
Extract the program from a LutLang::Lut contained in egraph
Sourcepub fn get_operand_classes(
&self,
_egraph: &EGraph<LutLang, LutAnalysis>,
) -> Result<Vec<Id>, String>
pub fn get_operand_classes( &self, _egraph: &EGraph<LutLang, LutAnalysis>, ) -> Result<Vec<Id>, String>
Extract the operand class ids from a LutLang::Lut contained in egraph
This filters out unwanted leaf nodes, like Programs.
Sourcepub fn get_lut_size(&self) -> Result<usize, String>
pub fn get_lut_size(&self) -> Result<usize, String>
Returns the fan-in of a LutLang::Lut
Sourcepub fn eval(
expr: &RecExpr<Self>,
inputs: &HashMap<String, bool>,
) -> Result<BitVec, String>
pub fn eval( expr: &RecExpr<Self>, inputs: &HashMap<String, bool>, ) -> Result<BitVec, String>
This funcion evaluates the expr as combinational logic under given inputs
Sourcepub fn deep_equals(&self, other: &Self, expr: &RecExpr<Self>) -> bool
pub fn deep_equals(&self, other: &Self, expr: &RecExpr<Self>) -> bool
Since variables/leaves can be duplicated in expressions, we sometimes need to do deep checks for equality.
This function returns true if the two nodes with children contained in expr are equal.
Sourcepub fn func_equiv(expr: &RecExpr<Self>, other: &RecExpr<Self>) -> Check
pub fn func_equiv(expr: &RecExpr<Self>, other: &RecExpr<Self>) -> Check
Given two expressions and a set of input values, this funcion returns true if they represent the same combinational logic
Sourcepub fn get_prim_name(&self) -> Option<String>
pub fn get_prim_name(&self) -> Option<String>
Returns the Verilog primitive name for a node type